@@ -7,8 +7,8 @@
 import jockey.handlers
 
 class NonfreeHandler(jockey.handlers.Handler):
-    def __init__(self, ui):
-        jockey.handlers.Handler.__init__(self, ui, 'Demo nonfree driver',
+    def __init__(self, backend):
+        jockey.handlers.Handler.__init__(self, backend, 'Demo nonfree driver',
             'This is just for demonstration and testing purposes.',
             'I will not do anything to your system.')
         self._enabled = False
@@ -26,9 +26,6 @@ class NonfreeHandler(jockey.handlers.Handler):
     def available(self):
         return True
 
-    def ui_category(self):
-        return self.ui._('Test driver')
-
     def enable(self):
         self._enabled = True
         self._used = True
