@@ -136,6 +136,7 @@ class Backend(dbus.service.Object):
         # cached D-BUS interfaces for _check_polkit_privilege()
         self.dbus_info = None
         self.polkit = None
+        self.main_loop = None
 
         self.enforce_polkit = True
         self._package_operation_in_progress = False
@@ -341,6 +342,8 @@ class Backend(dbus.service.Object):
         use, and new drivers which got available but are disabled.
         Mode can be "any" (default) to return all available drivers, or
         "free"/"nonfree" to select by license.
+
+        This will return empty lists if no package repositories are available.
         '''
         self._reset_timeout()
         self._check_polkit_privilege(sender, conn, 'com.ubuntu.devicedriver.check')
@@ -349,6 +352,10 @@ class Backend(dbus.service.Object):
             raise InvalidModeException(
                 'invalid mode %s: must be "free", "nonfree", or "any"' % mode)
 
+        if not self.has_repositories():
+            logging.warning('new_used_available(): No package repositories available, skipping check')
+            return ([], [])
+
         # read previously seen/used handlers
         seen = set()
         used = set()
@@ -530,6 +537,54 @@ class Backend(dbus.service.Object):
         '''
         return False
 
+    @dbus.service.signal(DBUS_INTERFACE_NAME)
+    def repository_update_progress(self, curr, total):
+        '''Report repository index update progress.
+
+        current and/or total might be -1 if time cannot be determined.
+        '''
+        return False
+
+    @dbus.service.method(DBUS_INTERFACE_NAME,
+        in_signature='', out_signature='b', sender_keyword='sender',
+        connection_keyword='conn')
+    def has_repositories(self, sender=None, conn=None):
+        '''Check if there are package respository indexes available.'''
+
+        return OSLib.inst.has_repositories()
+
+    @dbus.service.method(DBUS_INTERFACE_NAME,
+        in_signature='', out_signature='b', sender_keyword='sender',
+        connection_keyword='conn')
+    def update_repository_indexes(self, sender=None, conn=None):
+        '''Update package respository indexes
+        
+        Return True on success, False on failure.'''
+
+        self._reset_timeout()
+        self._check_polkit_privilege(sender, conn, 'com.ubuntu.devicedriver.check')
+
+        logging.debug('Updating repository indexes...')
+        OSLib.inst.update_repository_indexes(
+            hasattr(self, '_locations') and self.repository_update_progress or None)
+        if self.has_repositories():
+            logging.debug('... success')
+            return True
+        else:
+            logging.debug('... fail!')
+            return False
+
+    @dbus.service.method(DBUS_INTERFACE_NAME,
+        in_signature='', out_signature='', sender_keyword='sender',
+        connection_keyword='conn')
+    def shutdown(self, sender=None, conn=None):
+        '''Shut down the daemon.'''
+
+        logging.debug('Shutting down')
+        self._timeout = True
+        if self.main_loop:
+            self.main_loop.quit()
+
     #
     # Internal API for calling from Handlers (not exported through D-BUS)
     #
@@ -576,11 +631,11 @@ class Backend(dbus.service.Object):
         once the server is ready to take requests.
         '''
         dbus.service.Object.__init__(self, self.bus, '/DeviceDriver')
-        main_loop = gobject.MainLoop()
+        self.main_loop = gobject.MainLoop()
         self._timeout = False
         if timeout:
             def _t():
-                main_loop.quit()
+                self.main_loop.quit()
                 return True
             gobject.timeout_add(timeout * 1000, _t)
 
@@ -592,7 +647,7 @@ class Backend(dbus.service.Object):
         while not self._timeout:
             if timeout:
                 self._timeout = True
-            main_loop.run()
+            self.main_loop.run()
 
     @classmethod
     def create_dbus_server(klass, session_bus=False, handler_dir=None):
@@ -672,6 +727,12 @@ class Backend(dbus.service.Object):
         This initializes self.handlers as id → Handler map.'''
 
         self.handlers = {}
+
+        # shortcut if we do not have package repos
+        if not self.has_repositories():
+            logging.warning('_detect_handlers(): No package repositories available, skipping check')
+            return
+
         for h in detection.get_handlers(self,
                 self.driver_dbs,
                 handler_dir=self.handler_dir,
@@ -717,7 +778,8 @@ class Backend(dbus.service.Object):
         try:
             # we don't need is_challenge return here, since we call with AllowUserInteraction
             (is_auth, _, details) = self.polkit.CheckAuthorization(
-                    ('unix-process', {'pid': dbus.UInt32(pid, variant_level=1)}), 
+                    ('unix-process', {'pid': dbus.UInt32(pid, variant_level=1),
+                        'start-time': dbus.UInt64(0, variant_level=1)}), 
                     privilege, {'': ''}, dbus.UInt32(1), '', timeout=600)
         except dbus.DBusException, e:
             if e._dbus_error_name == 'org.freedesktop.DBus.Error.ServiceUnknown':
@@ -731,3 +793,4 @@ class Backend(dbus.service.Object):
             logging.debug('_check_polkit_privilege: sender %s on connection %s pid %i is not authorized for %s: %s' %
                     (sender, conn, pid, privilege, str(details)))
             raise PermissionDeniedByPolicy(privilege)
+
