@@ -769,7 +769,7 @@ def _driverid_to_handler(did, backend, mode):
         h = hclass(backend, *args, **kwargs)
     except Exception, e:
         logging.warning('could not instantiate handler class %s with args %s and kwargs %s: %s', 
-            str(hclass), str(args), str(kwargs), e.message)
+            str(hclass), str(args), str(kwargs), str(e))
         return None
 
     if 'description' in did.properties:
