@@ -377,13 +377,15 @@ class KernelModuleHandler(Handler):
     '''
     _loaded_modules = None
     
-    def __init__(self, backend, kernel_module, name=None, description=None, rationale=None):
+    def __init__(self, backend, kernel_module, name=None, description=None, rationale=None,
+                 do_blacklist=True):
         '''Create handler for a kernel module.
         
         If not given explicitly, the name is read from modinfo's 'description'
         field.
         '''
         self.module = kernel_module
+        self.do_blacklist = do_blacklist
         self._modinfo = detection.get_modinfo(self.module)
         if not name:
             assert self._modinfo, 'kernel module %s exists' % self.module
@@ -464,7 +466,8 @@ class KernelModuleHandler(Handler):
         This adds the module to the modprobe blacklist.
         '''
         Handler.disable(self)
-        OSLib.inst.blacklist_module(self.module, True)
+        if self.do_blacklist:
+            OSLib.inst.blacklist_module(self.module, True)
         self._modinfo = detection.get_modinfo(self.module)
         return False # TODO: can we make this automatic?
 
