@@ -17,7 +17,7 @@
 
 '''Encapsulate operations which are Linux distribution specific.'''
 
-import fcntl, os, subprocess, sys, logging, re, tempfile
+import fcntl, os, subprocess, sys, logging, re, tempfile, time
 from glob import glob
 
 class OSLib:
@@ -35,7 +35,10 @@ class OSLib:
         '''
         # relevant stuff for clients and backend
         self._get_os_version()
-        self.hal_get_property_path = '/usr/bin/hal-get-property'
+
+        # /sys/ path; the main purpose of changing this is for test
+        # suites, but some vendors might have /sys in a nonstandard place
+        self.sys_dir = '/sys'
 
         if client_only:
             return
@@ -44,10 +47,6 @@ class OSLib:
 
         # default paths
 
-        # /sys/ path; the main purpose of changing this is for test
-        # suites, but some vendors might have /sys in a nonstandard place
-        self.sys_dir = '/sys'
-
         # path to a modprobe.d configuration file where kernel modules are
         # enabled and disabled with blacklisting
         self.module_blacklist_file = '/etc/modprobe.d/blacklist-local.conf'
@@ -109,7 +108,7 @@ class OSLib:
         pkcon = subprocess.Popen(['pkcon', 'resolve', package],
             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         out = pkcon.communicate()[0]
-        return pkcon.returncode == 0 and '\ninstalled ' in out
+        return pkcon.returncode == 0 and '\ninstalled ' in out.lower()
 
     def package_description(self, package):
         '''Return a tuple (short_description, long_description) for a package.
@@ -230,9 +229,43 @@ class OSLib:
                 fail = True
 
         err += pkcon.stderr.read()
-        if pkcon.wait() != 0 or self.package_installed(package):
+        pkcon.wait()
+        if self.package_installed(package):
             raise SystemError, 'package %s failed to remove: %s' % (package, err) 
 
+    def has_repositories(self):
+        '''Check if package repositories are available.
+
+        This might not be the case after a fresh installation, when package
+        indexes haven't been downloaded yet.
+        '''
+        pkcon = subprocess.Popen(['pkcon', 'get-details', 'bash'],
+                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        out = pkcon.communicate()[0]
+        # PK can't detect package sizes without repositories
+        m = re.search("^\s*size:\s*0 bytes$", out, re.M)
+        return m == None
+
+    def update_repository_indexes(self, progress_cb):
+        '''Download package repository indexes.
+
+        Return True on success, False otherwise.
+
+        As this is called in the backend, this must happen noninteractively.
+        For progress reporting, progress_cb(current, total) is called
+        regularly. Passes '-1' for current and/or total if time cannot be
+        determined.
+        '''
+        pkcon = subprocess.Popen(['pkcon', 'refresh'],
+            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+
+        while pkcon.poll() == None:
+            time.sleep(0.3)
+            if progress_cb:
+                progress_cb(-1, -1)
+        pkcon.wait()
+        return self.has_repositories()
+
     def packaging_system(self):
         '''Return packaging system.
 
@@ -425,27 +458,18 @@ class OSLib:
 
         Either or both can be '' if they cannot be determined.
 
-        The default implementation queries hal.
+        The default implementation queries sysfs.
         '''
-
         try:
-            hal = subprocess.Popen([self.hal_get_property_path, '--udi',
-                '/org/freedesktop/Hal/devices/computer', '--key',
-                'system.hardware.vendor'], stdout=subprocess.PIPE,
-                close_fds=True)
-            vendor = hal.communicate()[0].strip()
-            assert hal.returncode == 0
-        except (OSError, AssertionError):
+            vendor = open(os.path.join(self.sys_dir,
+                'class', 'dmi', 'id', 'sys_vendor')).read().strip()
+        except IOError:
             vendor = ''
 
         try:
-            hal = subprocess.Popen([self.hal_get_property_path, '--udi',
-                '/org/freedesktop/Hal/devices/computer', '--key',
-                'system.hardware.product'], stdout=subprocess.PIPE,
-                close_fds=True)
-            product = hal.communicate()[0].strip()
-            assert hal.returncode == 0
-        except (OSError, AssertionError):
+            product = open(os.path.join(self.sys_dir,
+                'class', 'dmi', 'id', 'product_name')).read().strip()
+        except IOError:
             product = ''
 
         return (vendor, product)
