@@ -20,13 +20,18 @@ Concrete implementations need to implement a set of abstract presentation
 functions with an appropriate toolkit.
 '''
 
-import gettext, optparse, urllib2, tempfile, sys, time, os, threading
+import gettext, optparse, urllib2, tempfile, sys, time, os, threading, signal
 
 import gobject
 import dbus
 import dbus.service
 import dbus.mainloop.glib
 
+try:
+    import appindicator
+except:
+    pass
+
 from oslib import OSLib
 from backend import UnknownHandlerException, PermissionDeniedByPolicy, \
     BackendCrashError, convert_dbus_exceptions, \
@@ -80,6 +85,9 @@ class AbstractUI(dbus.service.Object):
         self.have_ui = False
         self.current_search = (None, None) # query, result
 
+        # make Control-C work properly
+        signal.signal(signal.SIGINT, signal.SIG_DFL)
+
     def backend(self):
         '''Return D-BUS backend client interface.
 
@@ -99,6 +107,7 @@ class AbstractUI(dbus.service.Object):
                     sys.exit(1)
                 else:
                     raise
+            self._check_repositories()
             self._call_progress_dialog(
                 self._('Searching for available drivers...'),
                 self._dbus_iface.detect, timeout=600)
@@ -110,6 +119,7 @@ class AbstractUI(dbus.service.Object):
                 if hasattr(e, '_dbus_error_name') and e._dbus_error_name == \
                     'org.freedesktop.DBus.Error.ServiceUnknown':
                     self._dbus_iface = Backend.create_dbus_client()
+                    self._check_repositories()
                     self._call_progress_dialog(
                         self._('Searching for available drivers...'),
                         self._dbus_iface.detect, timeout=600)
@@ -415,7 +425,9 @@ class AbstractUI(dbus.service.Object):
 
         # start the UI
         self.ui_show_main()
-        return self.ui_main_loop()
+        res = self.ui_main_loop()
+        self.backend().shutdown()
+        return res
 
     def list(self):
         '''Print a list of available handlers and their status to stdout.'''
@@ -494,6 +506,7 @@ class AbstractUI(dbus.service.Object):
             # we need to stay in the main loop so that the tray icon stays
             # around
             self.ui_main_loop()
+            self.backend().shutdown()
 
         return len(new_avail) > 0
 
@@ -529,6 +542,14 @@ class AbstractUI(dbus.service.Object):
         self.ui_progress_update(cur, total)
         self.ui_idle()
 
+    def _repository_progress_handler(self, cur, total):
+        if not self._repository_progress_shown:
+            self.ui_progress_start('', 
+                self._('Downloading and updating package indexes...'), total)
+            self._repository_progress_shown = True
+        self.ui_progress_update(cur, total)
+        self.ui_idle()
+
     def set_handler_enable(self, handler_id, action, confirm, gui=True):
         '''Enable, disable, or toggle a handler.
 
@@ -616,6 +637,12 @@ class AbstractUI(dbus.service.Object):
             return False
 
         newstate = bool(self.backend().handler_info(handler_id)['enabled'])
+
+        if enable and not newstate:
+            self.error_message('', '%s\n\n%s: /var/log/jockey.log' % (
+                self._('Sorry, installation of this driver failed.'),
+                self._('Please have a look at the log file for details')))
+
         return i['enabled'] != newstate
 
     def _get_description_rationale_text(self, h_info):
@@ -752,6 +779,29 @@ class AbstractUI(dbus.service.Object):
 
         return hwid
 
+    def _check_repositories(self):
+        '''Check if we have package repositories, and if not, offer to update.'''
+
+        if self._dbus_iface.has_repositories():
+            return
+
+        success = False
+        try:
+            self._repository_progress_shown = False
+            success = convert_dbus_exceptions(dbus_sync_call_signal_wrapper,
+                    self._dbus_iface, 'update_repository_indexes',
+                    {'repository_update_progress':
+                        self._repository_progress_handler})
+        finally:
+            if self._repository_progress_shown:
+                self.ui_progress_finish()
+
+        # check success
+        if not success:
+            self.error_message('', self._(
+    'Downloading package indexes failed, please check your network status. '
+    'Most drivers will not be available.'))
+
     #
     # Session D-BUS server methods
     # 
@@ -886,8 +936,20 @@ class AbstractUI(dbus.service.Object):
 
         This should preferably create a tray icon. Clicking on the tray icon or
         notification should run the GUI.
+
+        This method will attempt to instantiate an appindicator to return to
+        both the GTK and KDE children.  If whatever reason it fails (missing
+        python-appindicator) then it should behave as it did before.
         '''
-        raise NotImplementedError, 'subclasses need to implement this'
+        try:
+            indicator = appindicator.Indicator('jockey', 'jockey',
+                    appindicator.CATEGORY_HARDWARE)
+        except Exception, e:
+            raise NotImplementedError, 'appindicator support not available: %s' \
+                '\nsubclasses need to implement this' % str(e)
+        
+        indicator.set_status(appindicator.STATUS_ATTENTION)
+        return indicator
 
     def ui_idle(self):
         '''Process pending UI events and return.
