@@ -35,7 +35,7 @@ class XorgDriverHandler(KernelModuleHandler):
     def __init__(self, backend, module, driver_package, xorg_driver,
         alt_free_driver, extra_conf_options={}, add_modules=[],
             disable_modules=[], remove_modules=[], name=None, description=None,
-            rationale=None):
+            rationale=None, do_blacklist=True):
         '''Create handler for a particular X.org graphics driver.
         
         This usually consists of a kernel module and a driver package, plus
@@ -53,7 +53,7 @@ class XorgDriverHandler(KernelModuleHandler):
           section of xorg.conf when enabling this driver (and added when disabling)
         '''
         KernelModuleHandler.__init__(self, backend, module, name, description,
-            rationale)
+            rationale, do_blacklist=do_blacklist)
         self.package = driver_package
         self.xorg_driver = xorg_driver
         self.alt_free_driver = alt_free_driver
@@ -156,9 +156,11 @@ class XorgDriverHandler(KernelModuleHandler):
 
         KernelModuleHandler.enable(self)
 
-        # do not mangle xorg.conf if package installation has been aborted
-        if not OSLib.inst.package_installed(self.package):
-            return
+        # do not mangle xorg.conf if package installation has been aborted or
+        # module is not available (e. g. because DKMS did not build it)
+        if not KernelModuleHandler.enabled(self):
+            logging.error('XorgDriverHandler.enable(): package or module not installed, aborting')
+            return False
         
         # see if the same proprietary driver is in use
         # in the xorg.conf. If so do not create a backup
