@@ -121,6 +121,7 @@ class DbusBackendTest(sandbox.LogTestCase):
                 'used': 'False',
                 'changed': 'False',
                 'recommended': 'False',
+                'announce': 'True',
             })
 
     def test_set_enabled(self):
@@ -331,6 +332,7 @@ class LocalBackendTest(sandbox.LogTestCase):
                 'changed': 'False',
                 'package': 'mesa-vanilla',
                 'recommended': 'False',
+                'announce': 'True',
             })
 
     def test_handler_files(self):
@@ -483,7 +485,7 @@ class LocalBackendTest(sandbox.LogTestCase):
         # ... and add it to set of available handlers
         self.assert_('printer:openprinting-ppds-postscript-hp:HP' not in b.available())
         self.assertEqual(b.search_driver(
-            'printer_deviceid:MFG:Hewlett-Packard;MDL:HP LaserJet 3020'),
+            'printer_deviceid:MFG:Hewlett-Packard;MDL:HP DesignJet 3500CP'),
             ['printer:openprinting-ppds-postscript-hp:Postscript-HP'])
         self.assert_('printer:openprinting-ppds-postscript-hp:Postscript-HP' in b.available())
 
