@@ -51,8 +51,7 @@ class DetectionTest(sandbox.LogTestCase):
 
         shutil.rmtree(OSLib.inst.handler_dir)
         os.mkdir(OSLib.inst.handler_dir)
-        # reset to default
-        OSLib.inst.hal_get_property_path = OSLib.inst.hal_get_property_path_vapor
+        OSLib.inst.reset_dmi()
         # remove DriverDB caches
         for f in glob(os.path.join(OSLib.inst.backup_dir, 'driverdb-*.cache')):
             os.unlink(f)
@@ -425,7 +424,7 @@ class PickyHandler(jockey.handlers.KernelModuleHandler):
 
         backend = jockey.backend.Backend()
         # should *not* contact server, do that before starting server
-        OSLib.inst.hal_get_property_path = OSLib.inst.hal_get_property_path_empty
+        OSLib.inst.set_dmi('', '')
         db = jockey.detection.XMLRPCDriverDB('http://localhost:80080')
 
         # until we update, there is no data
@@ -492,8 +491,13 @@ kmod:spam([KernelModuleHandler, free, enabled] free mystical module without moda
         finally:
             sandbox.stop_driverdb_server()
 
-        result = jockey.detection.get_handlers(backend, db,
-            hardware=backend.hardware)
+        orig_getlocale = locale.getlocale
+        try:
+            locale.getlocale = lambda c: ('en_US', 'UTF-8')
+            result = jockey.detection.get_handlers(backend, db,
+                hardware=backend.hardware)
+        finally:
+            locale.getlocale = orig_getlocale
         out = '\n'.join(sorted([str(h) for h in result]))
         self.assertEqual(out, 
 '''firmware:firmwifi([FirmwareHandler, nonfree, disabled] standard module which needs firmware)
@@ -764,7 +768,7 @@ kmod:vanilla3d([VanillaGfxHandler, nonfree, enabled] nonfree, replacement graphi
         self.assertEqual(drv['package'], 'openprinting-ppds-postscript-hp')
         self.assertEqual(drv['driver_vendor'], 'HP')
         self.assertEqual(drv['free'], True)
-        self.assert_('postscript-hp/rpms' in drv['repository'].lower())
+        self.assert_('printdriver/rpms' in drv['repository'].lower())
         self.assert_('license' in drv)
 
     def test_openprinting_unknownpkg(self):
