@@ -18,7 +18,7 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-import unittest, os
+import unittest, os, tempfile, shutil
 
 from jockey.oslib import OSLib
 import sandbox
@@ -26,9 +26,6 @@ import sandbox
 class OSLibTest(sandbox.LogTestCase):
     '''OSLib tests'''
 
-    def tearDown(self):
-        OSLib.inst.hal_get_property_path = OSLib.inst.hal_get_property_path_vapor
-
     def test_module_blacklisting(self):
         '''module_blacklisted() and blacklist_module()'''
 
@@ -115,28 +112,18 @@ class OSLibTest(sandbox.LogTestCase):
     def test_get_system_vendor_product(self):
         '''get_system_vendor_product()'''
 
-        # with existing system name
-        v, p = OSLib.inst.get_system_vendor_product()
-        self.assertEqual(v, 'VaporTech Inc.')
-        self.assertEqual(p, 'Virtualtop X-1')
+        # vendor/product name
+        self.assertEqual(OSLib.inst.get_system_vendor_product(), 
+                ('VaporTech Inc.', 'Virtualtop X-1'))
 
-        # without existing system name
-        OSLib.inst.hal_get_property_path = OSLib.inst.hal_get_property_path_empty
-        v, p = OSLib.inst.get_system_vendor_product()
-        self.assertEqual(v, '')
-        self.assertEqual(p, '')
+        # without vendor/product name
+        OSLib.inst.set_dmi(None, None)
+        self.assertEqual(OSLib.inst.get_system_vendor_product(), ('', ''))
 
-        # with failing hal-get-property
-        OSLib.inst.hal_get_property_path = OSLib.inst.hal_get_property_path_fail
-        v, p = OSLib.inst.get_system_vendor_product()
-        self.assertEqual(v, '')
-        self.assertEqual(p, '')
-
-        # with nonexisting hal-get-property
-        OSLib.inst.hal_get_property_path = '/nonexisting'
-        v, p = OSLib.inst.get_system_vendor_product()
-        self.assertEqual(v, '')
-        self.assertEqual(p, '')
+        # vendor name only
+        OSLib.inst.set_dmi('VaporTech Inc.', None)
+        self.assertEqual(OSLib.inst.get_system_vendor_product(), 
+            ('VaporTech Inc.', ''))
 
         # actual system's implementation is a string
         v, p = OSLib(client_only=True).get_system_vendor_product()
@@ -194,3 +181,12 @@ class OSLibTest(sandbox.LogTestCase):
         self.assert_(o.package_installed(test_package))
         o.remove_package(test_package, None)
         self.failIf(o.package_installed(test_package))
+
+    def test_has_repositories(self):
+        '''has_repositories()
+
+        This is only a shallow test that the function works. We assume that we
+        have repositories in the test environment for now.
+        '''
+        o = OSLib()
+        self.assertEqual(o.has_repositories(), True)
