@@ -20,7 +20,9 @@ for self tests.'''
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-import tempfile, atexit, shutil, os, os.path, sys
+import tempfile, atexit, shutil, os, os.path, sys, signal, subprocess
+import unittest, xmlrpclib, time, socket, traceback
+from SimpleXMLRPCServer import SimpleXMLRPCServer
 
 from jockey.oslib import OSLib
 from jockey.detection import HardwareID, DriverID, DriverDB
@@ -59,7 +61,7 @@ fake_modinfo = {
     },
     'spam': {
         'filename': '/lib/modules/0.8.15/extra/spam.ko',
-        'license': 'GPL',
+        'license': 'GPL v2',
         'description': 'free mystical module without modaliases',
     },
     'dullstd': {
@@ -142,72 +144,148 @@ fake_sys = {
 fake_db = {
     # vanilla3d option for the graphics card
     HardwareID('modalias', 'pci:v00001001d00003D*sv*sd*bc03sc*i*'): {
-        ('Foonux', '42'): [DriverID(handler='VanillaGfxHandler')],
-        ('Foonux', '41'): [DriverID(handler='IShallNotExist')],
-        ('RedSock', '2.0'): [DriverID(handler='Vanilla3DHandler',
-            repository='http://nonfree.redsock.com/addons')]
+        ('', ''): {
+            ('Foonux', '42'): [DriverID(jockey_handler='VanillaGfxHandler'),
+                DriverID(foobar='IHaveNoHandler')],
+            ('Foonux', '41'): [DriverID(jockey_handler='IShallNotExist')],
+            ('RedSock', '2.0'): [DriverID(jockey_handler='Vanilla3DHandler',
+                repository='http://nonfree.redsock.com/addons')]
+        }
     },
 
-    # driver for the unknown piece of hardware; test multiple handlers here
+    # driver for the unknown piece of hardware; test multiple handlers here, as
+    # well as multiple system IDs
     HardwareID('modalias', 'pci:v0000FFF0d0000*sv*sd*bc06sc01i*'): {
-        ('Foonux', '42'): [
-            DriverID(handler='KernelModuleHandler', module='spam'),
-            DriverID(handler='KernelModuleHandler', module='mint')
-        ],
+        ('', ''): {
+            ('Foonux', '42'): [
+                DriverID(jockey_handler='KernelModuleHandler', kernel_module='spam'),
+                DriverID(jockey_handler='KernelModuleHandler', kernel_module='mint'),
+                # entries without a driver type are invalid and should be
+                # ignored; since our fake XMLRPC server always adds it by
+                # default, we tell it not to
+                DriverID(no_driver_type_tag='1', driver_vendor='Foo'),
+                # unknown driver types should be ignored
+                DriverID(driver_type='unknown_type', kernel_module='chocolate', driver_vendor='Bar'),
+                # uninstantiable drivers should be ignored
+                DriverID(jockey_handler='KernelModuleHandler',
+                    kernel_module='chocolate', unknown_property='moo'),
+            ],
+        },
+        ('VaporTech Inc.', 'Virtualtop X-1'): {
+            ('Foonux', '42'): [
+                DriverID(jockey_handler='KernelModuleHandler', kernel_module='chocolate',
+                    repository='http://vaportech.com/x1',
+                    package='spam-x1', driver_vendor='VaporTech',
+                    description={'C': 'VaporTech choc handler', 'de_DE':
+                        'German description', 'cs': 'Czech description'},
+                    long_description={'C': 'Multi\nline'},
+                    version='1.1-foo42-vt3'),
+                # test correct cloning of custom and standard handlers
+                DriverID(jockey_handler='NoDetectMod', description={'C': 'WhiteHat ndmod'}, driver_vendor='White Hat'),
+                DriverID(jockey_handler='NoDetectMod', description={'C': 'BlackHat ndmod'}, driver_vendor='Black Hat'),
+                DriverID(jockey_handler='KernelModuleHandler', kernel_module='mint',
+                    description={'C': 'UbuOne mintmod'}, driver_vendor='UbuOne'),
+                DriverID(jockey_handler='KernelModuleHandler', kernel_module='mint',
+                    description={'C': 'UbunTwo mintmod'},
+                    driver_vendor='UbunTwo', recommended=True),
+                # unknown kmod, need to specify license and description
+                # ourselves
+                DriverID(jockey_handler='KernelModuleHandler', kernel_module='apple',
+                    description={'C': 'Unknown local kmod'}, package='pretzel',
+                    driver_vendor='Apple', free=False, license='Kills kittens'),
+            ],
+
+        },
+        ('VaporTech Inc.', 'Virtualtop Z-3'): {
+            ('Foonux', '42'): [
+                DriverID(jockey_handler='KernelModuleHandler', kernel_module='spam',
+                    repository='http://vaportech.com/z3',
+                    package='spam-z3', driver_vendor='VaporTech')
+            ],
+        },
     },
 
     # driver for the second wifi, test firmware handler
     HardwareID('modalias', 'fire:1.2'): {
-        ('Foonux', '42'): [
-            DriverID(handler='FirmwareHandler', module='firmwifi',
-                testfile='/lib/firmware/firmwifi.bin', url='http://foo.bar'),
-        ],
+        ('', ''): {
+            ('Foonux', '42'): [
+                DriverID(jockey_handler='FirmwareHandler', kernel_module='firmwifi',
+                    testfile='/lib/firmware/firmwifi.bin',
+                    url='http://foo.bar', recommended=True),
+            ],
+        },
     },
 
     # unknown driver
     HardwareID('pci', '9876/FEDC'): {
-        ('Foonux', '42'): [DriverID(handler='BogusHandler')],
-        ('Oobuntu', 'Eccentric Emu'): [DriverID(handler='BogusHandler')],
+        ('', ''): {
+            ('Foonux', '42'): [DriverID(jockey_handler='KernelModuleHandler',
+                kernel_module='spam')],
+            ('Oobuntu', 'Eccentric Emu'): [DriverID(jockey_handler='BogusHandler')],
+        },
     }
 }
 
 fake_pkginfo = {
-    'mesa-vanilla': {
-        'description': ('X.org libraries for the Vanilla 3D driver', 
-            'This package provides optimized Mesa libraries for the Vanilla '
-            'graphics cards.'),
-        'free': False,
+    'default': {
+        'mesa-vanilla': {
+            'description': ('X.org libraries for the Vanilla 3D driver', 
+                'This package provides optimized Mesa libraries for the Vanilla '
+                'graphics cards.'),
+            'free': False,
+            'files': ['/bin/mesa-vanilla-test', '/lib/X11/vesa.drv'],
+        },
+        'mesa-std': {
+            'description': ('standard system mesa libraries',
+                'default mesa libs for free drivers'),
+            'free': True,
+        },
+        'coreutils': {
+            'description': ('unrelated system package',
+                'This package should always be installed.'),
+            'free': True,
+        },
+        'pretzel': {
+            'description': ('yet another test packge',
+                'This is yummy, but uninteresting.'),
+            'free': False,
+        },
     },
-    'mesa-std': {
-        'description': ('standard system mesa libraries',
-            'default mesa libs for free drivers'),
-        'free': True,
-    },
-    'coreutils': {
-        'description': ('unrelated system package',
-            'This package should always be installed.'),
-        'free': True,
-    },
-    'pretzel': {
-        'description': ('yet another test packge',
-            'This is yummy, but uninteresting.'),
-        'free': False,
+    'http://vaportech.com/x1': {
+        'spam-x1': {
+            'description': ('SpamX1 driver', 'spamx1'),
+            'free': True,
+        },
+        'foo-y1': {
+            'description': ('FooY1 driver', 'fooy1'),
+            'free': True,
+        },
     },
 }
 
 #-------------------------------------------------------------------#
 
+class LogTestCase(unittest.TestCase):
+    '''TestCase which logs method names to sandbox.log.'''
+
+    def run(self, result=None):
+        print >> log, '--- Running test', self.id()
+        unittest.TestCase.run(self, result)
+
+#-------------------------------------------------------------------#
+
 class TestOSLib(OSLib):
     '''Test suite implementation of OSLib.
     
     This builds a realistic mini-chroot and fake module/package system.
     '''
     def __init__(self):
-        OSLib.__init__(self)
-
         # set up a fake environment
         self.workdir = tempfile.mkdtemp()
         atexit.register(shutil.rmtree, self.workdir)
+
+        OSLib.__init__(self)
+
         self._make_modinfo()
         self._make_proc_modules()
         self._make_modprobe()
@@ -215,22 +293,49 @@ class TestOSLib(OSLib):
         self._make_sys()
         self._make_xorg_conf()
 
-        self.module_blacklist_file = os.path.join(self.workdir, 'module-blacklist')
+        self.reset_packages()
+        self.reset_dmi()
+
+        os.mkdir(os.path.join(self.workdir, 'modules.d'))
+        self.module_blacklist_file = os.path.join(self.workdir, 'modules.d', 'module-blacklist')
 
         self.handler_dir = os.path.join(self.workdir, 'handlers')
         os.mkdir(self.handler_dir)
         self.check_cache = os.path.join(self.workdir, 'check_cache')
-        self.backup_dir = os.path.join(self.workdir, 'backup')
-        os.mkdir(self.backup_dir)
-
-        self.installed_packages = set(['mesa-std', 'coreutils'])
-        self.next_package_install_fails = False
-
-        self.reboot_flag = False
 
         self.help_available = False
         self.help_called = False
 
+    def reset_packages(self):
+        '''Reset repositories and packages.'''
+
+        self.installed_packages = set(['mesa-std', 'coreutils'])
+        self.next_package_install_fails = False
+        self.repositories = set(['default'])
+        # set this to a string to raise it on next install/remove_package()
+        self.pending_install_remove_exception = None
+
+    def reset_dmi(self):
+        '''Reset system vendor/product to default.'''
+
+        self.set_dmi('VaporTech Inc.', 'Virtualtop X-1')
+
+    def set_dmi(self, vendor, product):
+        '''Set system vendor/product strings.'''
+
+        dmipath = os.path.join(self.sys_dir, 'class', 'dmi', 'id')
+        if os.path.isdir(dmipath):
+            shutil.rmtree(dmipath)
+        os.makedirs(dmipath)
+        if vendor is not None:
+            f = open(os.path.join(dmipath, 'sys_vendor'), 'w')
+            f.write(vendor + '\n')
+            f.close()
+        if product is not None:
+            f = open(os.path.join(dmipath, 'product_name'), 'w')
+            f.write(product + '\n')
+            f.close()
+
     def _make_modinfo(self):
         '''Create a dummy modinfo program which outputs the fake_modinfo data
         and set self.modinfo_path.
@@ -426,6 +531,10 @@ EndSection
         self.os_vendor = 'Foonux'
         self.os_version = '42'
 
+    def set_backup_dir(self):
+        self.backup_dir = os.path.join(self.workdir, 'backup')
+        os.mkdir(self.backup_dir)
+
     def ignored_modules(self):
         return set(['dullstd'])
 
@@ -435,58 +544,138 @@ EndSection
         
         This should raise a ValueError if the package is not available.'''
 
-        try:
-            return fake_pkginfo[package]['description']
-        except KeyError:
-            raise ValueError, 'no such package'
+        for repo in self.repositories:
+            try:
+                return fake_pkginfo[repo][package]['description']
+            except KeyError:
+                continue
+        raise ValueError, 'no such package'
 
     def is_package_free(self, package):
         '''Return if given package is free software.'''
 
-        return fake_pkginfo[package]['free']
+        for repo in self.repositories:
+            try:
+                return fake_pkginfo[repo][package]['free']
+            except KeyError:
+                continue
+        raise ValueError, 'no such package'
 
     def package_installed(self, package):
         '''Return if the given package is installed.'''
 
         return package in self.installed_packages
 
-    def install_package(self, package, ui):
+    def package_files(self, package):
+        '''Return a list of files shipped by a package.
+        
+        This should raise a ValueError if the package is not installed.
+        '''
+        if self.package_installed(package):
+            for repo in self.repositories:
+                try:
+                    return fake_pkginfo[repo][package]['files']
+                except KeyError:
+                    continue
+        raise ValueError, 'package not installed'
+
+    def install_package(self, package, progress_cb):
         '''Install the given package.
 
-        The current UI object is passed as well, in case package installation
-        wants to do some callbacks/confirmation dialogs/queries.
+        As this is called in the backend, this must happen noninteractively.
+        For progress reporting, progress_cb(phase, current, total) is called
+        regularly, with 'phase' being 'download' or 'install'. If the callback
+        returns True, the installation is attempted to get cancelled (this
+        will probably succeed in the 'download' phase, but not in 'install').
+        Passes '-1' for current and/or total if time cannot be determined.
 
         If this succeeds, subsequent package_installed(package) calls must
-        return True.'''
+        return True.
+
+        Any installation failure should be raised as a SystemError.
+        '''
+        for repo in self.repositories:
+            if package in fake_pkginfo[repo]:
+                break
+        else:
+            raise ValueError, 'no such package'
+
+        if self.pending_install_remove_exception:
+            e = self.pending_install_remove_exception
+            self.pending_install_remove_exception = None
+            raise SystemError, e
 
-        assert package in fake_pkginfo
         if self.next_package_install_fails:
             self.next_package_install_fails = False
         else:
             self.installed_packages.add(package)
 
-    def remove_package(self, package, ui):
+        # bogus progress
+        if progress_cb:
+            progress_cb('download', 1, 2)
+            time.sleep(0.2)
+            progress_cb('download', 2, 2)
+            time.sleep(0.2)
+            progress_cb('install', 1, 3)
+            time.sleep(0.2)
+            progress_cb('install', 3, 3)
+
+    def remove_package(self, package, progress_cb):
         '''Uninstall the given package.
 
-        The current UI object is passed as well, in case package installation
-        wants to do some callbacks/confirmation dialogs/queries.
+        As this is called in the backend, this must happen noninteractively.
+        For progress reporting, progress_cb(current, total) is called
+        regularly. Passes '-1' for current and/or total if time cannot be
+        determined.
 
         If this succeeds, subsequent package_installed(package) calls must
-        return False.'''
+        return False.
+
+        Any removal failure should be raised as a SystemError.
+        '''
+        if self.pending_install_remove_exception:
+            e = self.pending_install_remove_exception
+            self.pending_install_remove_exception = None
+            raise SystemError, e
 
-        assert package in fake_pkginfo
         self.installed_packages.remove(package)
 
-    def ui_notify_reboot(self, ui):
-        '''Indicate that the user should do a reboot to activate changes (such
-        as changing an X.org video driver).'''
+        # bogus progress
+        if progress_cb:
+            progress_cb(1, 2)
+            time.sleep(0.2)
+            progress_cb(2, 2)
 
-        self.reboot_flag = True
+    def add_repository(self, repository):
+        '''Add a repository.
 
-    def pop_reboot_flag(self):
-        result = self.reboot_flag
-        self.reboot_flag = False
-        return result
+        The format for repository is distribution specific. This function
+        should also download/update the package index for this repository.
+
+        This should throw a ValueError if the repository is invalid or
+        inaccessible.
+        '''
+
+        if repository not in fake_pkginfo:
+            raise ValueError, 'repository %s does not exist' % repository
+
+        self.repositories.add(repository)
+
+    def remove_repository(self, repository):
+        '''Remove a repository.
+
+        The format for repository is distribution specific.
+        '''
+
+        try:
+            self.repositories.remove(repository)
+        except KeyError:
+            pass
+
+    def repository_enabled(self, repository):
+        '''Check if given repository is enabled.'''
+
+        return repository in self.repositories
 
     def ui_help_available(self, ui):
         '''Return if help is available.
@@ -505,14 +694,6 @@ EndSection
         '''
         self.help_called = True
 
-    def open_app(self, ui, custom_args = None):
-        l = open(os.path.join(self.workdir, 'open_app.log'), 'w')
-        if custom_args:
-            print >> l, ' '.join(custom_args)
-        else:
-            print >> l, ' '.join(sys.argv)
-        l.close()
-
 #-------------------------------------------------------------------#
 
 class AllAvailOSLib(OSLib):
@@ -524,16 +705,22 @@ class AllAvailOSLib(OSLib):
     '''
 
     def __init__(self):
+        self.workdir = tempfile.mkdtemp()
+
         OSLib.__init__(self)
         self.installed_packages = set()
         self.blacklisted_modules = set()
 
-        self.workdir = tempfile.mkdtemp()
         atexit.register(shutil.rmtree, self.workdir)
 
-        self.reboot_flag = False
         self._make_modinfo()
         self.xorg_conf_path = os.path.join(self.workdir, 'xorg.conf')
+        self.module_blacklist_file = os.path.join(self.workdir, 
+            'module-blacklist')
+
+    def set_backup_dir(self):
+        self.backup_dir = os.path.join(self.workdir, 'backup')
+        os.mkdir(self.backup_dir)
 
     def package_description(self, package):
         '''Return a tuple (short_description, long_description) for a package.
@@ -552,12 +739,12 @@ class AllAvailOSLib(OSLib):
 
         return package in self.installed_packages
 
-    def install_package(self, package, ui):
+    def install_package(self, package, progress_cb):
         '''Install the given package.'''
 
         self.installed_packages.add(package)
 
-    def remove_package(self, package, ui):
+    def remove_package(self, package, progress_cb):
         '''Uninstall the given package.'''
 
         self.installed_packages.remove(package)
@@ -582,13 +769,6 @@ class AllAvailOSLib(OSLib):
             except KeyError:
                 pass
 
-    def ui_notify_reboot(self, ui):
-        '''Indicate that the user should do a reboot to activate changes (such
-        as changing an X.org video driver).'''
-
-        self.reboot_flag = True
-
-
     def _make_modinfo(self):
         '''Create a dummy modinfo program which outputs dummy data
         and set self.modinfo_path.
@@ -619,12 +799,27 @@ print '%%-16s%%s' %% ('description:', '%%s module' %% m)
 #-------------------------------------------------------------------#
 
 class TestDriverDB(DriverDB):
-    '''Test suite implementation of DriverDB'''
+    '''Test suite implementation of DriverDB.
+    
+    This uses fake_db, but only the default system ID.'''
 
-    def query(self, hwid):
-        return fake_db.get(hwid, {}).get(
+    # TODO: drop self.updated, use XMLRPCDB for that test
+    def __init__(self):
+        DriverDB.__init__(self)
+        self.updated = False
+
+    def _do_query(self, hwid):
+        if not self.updated:
+            return []
+
+        return fake_db.get(hwid, {}).get(('', ''), {}).get(
             (OSLib.inst.os_vendor, OSLib.inst.os_version), [])
 
+    def _do_update(self, hwids):
+        '''Pretend that fake_db is acquired from a remote server.'''
+
+        self.updated = True
+
 #-------------------------------------------------------------------#
 
 class TestUI(AbstractUI):
@@ -634,8 +829,16 @@ class TestUI(AbstractUI):
         self.confirm_response = None
         self.notification_stack = []
         self.main_loop_active = False
-        self.cur_download = [None, None, None] # (url, size, total)
-        self.cancel_download = False # whether to cancel the next download at 30%
+        self.cur_progress = [None, None, None] # (desc, size, total)
+        self.cancel_progress = False # whether to cancel the next progress dialog at 30%
+
+    def backend(self):
+        if self._dbus_iface is None:
+            import jockey.backend as b
+            # use a local backend here for simplicity; D-BUS operation is
+            # already exercised elsewhere in the test suite
+            self._dbus_iface = b.Backend()
+        return self._dbus_iface
 
     def convert_keybindings(self, str):
         return str
@@ -643,6 +846,9 @@ class TestUI(AbstractUI):
     def ui_init(self):
         pass
 
+    def ui_show_main(self):
+        pass
+
     def ui_main_loop(self):
         '''Main loop for the user interface.
         
@@ -650,6 +856,7 @@ class TestUI(AbstractUI):
         the exit code.'''
 
         self.main_loop_active = True
+        return 0
 
     def ui_idle(self):
         pass
@@ -685,43 +892,142 @@ class TestUI(AbstractUI):
 
         return self.notification_stack.pop()
 
-    def ui_download_start(self, url, total_size):
-        '''Create a progress dialog for a download of given URL.
+    def ui_progress_start(self, title, description, total):
+        '''Create a progress dialog.'''
 
-        total_size specifes the number of bytes to download, or -1 if it cannot
-        be determined. In this case the dialog should display an indeterminated
-        progress bar (bouncing back and forth).'''
+        assert description
+        self.cur_progress = [description, 0, total]
 
-        assert url
-        self.cur_download = [url, 0, total_size]
-
-    def ui_download_progress(self, current_size, total_size):
-        '''Update download progress of current download.
+    def ui_progress_update(self, current, total):
+        '''Update status of current progress dialog.
         
-        This should return True to cancel the current download, and False
-        otherwise.'''
+        current/total specify the number of steps done and total steps to
+        do, or -1 if it cannot be determined. In this case the dialog should
+        display an indeterminated progress bar (bouncing back and forth).
 
-        assert self.cur_download[0]
-        assert self.cur_download[1] is not None
-        assert self.cur_download[2] is not None
-        assert current_size > self.cur_download[1]
-        assert total_size == self.cur_download[2]
+        This should return True to cancel, and False otherwise.
+        '''
+        assert self.cur_progress[0]
+        assert self.cur_progress[1] is not None
+        assert self.cur_progress[2] is not None
+        assert (current > self.cur_progress[1]) or current == -1
+        assert total == self.cur_progress[2]
 
-        self.cur_download[1] = current_size
+        self.cur_progress[1] = current
 
-        return self.cancel_download and float(current_size)/total_size >= 0.3
+        return self.cancel_progress and float(current)/total >= 0.3
 
-    def ui_download_finish(self):
-        '''Close the current download progress dialog.'''
+    def ui_progress_finish(self):
+        '''Close the current progress dialog.'''
+
+        self.cancel_progress = False
+
+#-------------------------------------------------------------------#
+
+def xmlrpc_driverdb_query(proto_ver, proto_subver, request):
+    '''XML-RPC server query() implementation.
+
+    This conforms to protocol version 20080407/0, which is the only defined
+    protocol version at the moment.
+
+    Example:
+    query('20080407', '0', {
+          'components': ['modalias:pci:crap', 'printer:Canon_BJ2'],
+          'system_vendor': 'Dell',
+          'system_product': 'Latitude D430',
+          'os_name': 'RedSock',
+          'os_version': '2.0',
+          'kernel_ver': '2.6.24-15-generic',
+          'architecture': 'i686'}) =
+      ('20080407', '0', {'modalias:pci:crap': [dr_crap1, dr_crap2], 'printer:Canon_BJ2': [dr_pr1]})
+    '''
+    if proto_ver != '20080407':
+        return ('20080407', '0', {})
+
+    result = {}
+    for comp_id in request.get('components', []):
+        (type, val) = comp_id.split(':', 1)
+        try:
+            hw_entry = fake_db.get(HardwareID(type, val), {})
+            system_entry = hw_entry.get((request['system_vendor'],
+                request['system_product']), hw_entry.get(('', ''), {}))
+            drivers = system_entry.get((request['os_name'], request['os_version']), [])
+        except KeyError:
+            break # invalid request
+
+        result[comp_id] = []
+
+        for did in drivers:
+            props = did.properties.copy()
+            if 'driver_type' not in props and 'no_driver_type_tag' not in props:
+                props['driver_type'] = 'kernel_module'
+            if props.get('jockey_handler') == 'KernelModuleHandler':
+                del props['jockey_handler']
+            if 'description' not in props:
+                if 'kernel_module' in props:
+                    props['description'] = {'C': fake_modinfo[props['kernel_module']]['description']}
+                else:
+                    props['description'] = {'C': 'desc'}
+
+            result[comp_id].append(props)
+
+    return ('20080407', '0', result)
+
+def xmlrpc_driverdb_query_bogus(proto_ver, proto_subver, request):
+    '''XML-RPC server query() implementation with bogus protocol.'''
+
+    return ('20070301', '2', {'modalias:pci:crap': ['bogus']})
+
+def start_driverdb_server(correct_protocol=True):
+    '''Create XML-RPC driver db server.
+
+    This will listen on localhost:80080 and serve fake_db. This must be
+    terminated with stop_driverdb_server().
+
+    If correct_protocol is False, this will answer with a bogus protocol
+    version.
+    '''
+
+    start_driverdb_server.pid = os.fork()
+    if start_driverdb_server.pid == 0:
+        try:
+            s = SimpleXMLRPCServer(('localhost', 80080), logRequests=False)
+            s.register_introspection_functions()
+            if correct_protocol:
+                s.register_function(xmlrpc_driverdb_query, 'query')
+            else:
+                s.register_function(xmlrpc_driverdb_query_bogus, 'query')
+            s.serve_forever()
+        except:
+            print >> sys.stderr, '********** DEMO XML-RPC server failed: ***********'
+            traceback.print_exc()
+            os._exit(1)
+
+    # wait until it is ready
+    c = xmlrpclib.ServerProxy('http://localhost:80080')
+    timeout = 100
+    while timeout > 0:
+        time.sleep(0.1)
+        try:
+            c.query('0', '0', {})
+        except socket.error:
+            timeout -= 1
+            continue
+        break
+    if timeout == 0:
+        raise SystemError, 'XML-RPC demo server did not start'
+
+def stop_driverdb_server():
+    '''Stop the XML-RPC driver db server.'''
+
+    os.kill(start_driverdb_server.pid, signal.SIGTERM)
+    os.wait()
 
-        self.cancel_download = False
 
 #-------------------------------------------------------------------#
 # TestOSLib consistency tests
 
-import unittest, subprocess
-
-class TestOSLibConsistencyTest(unittest.TestCase):
+class TestOSLibConsistencyTest(LogTestCase):
     def test_modinfo_output(self):
         '''test suite's modinfo output for known modules'''
         
@@ -797,6 +1103,7 @@ license:        GPL
         finally:
             open(OSLib.inst.proc_modules, 'w').write(orig_content)
 
+
     def test_sys_symlinks(self):
         '''all symlinks in fake /sys are valid'''
 
@@ -836,6 +1143,159 @@ alias pci:v00001001d00003D*sv*sd*bc03sc*i* vanilla3d
         self.failIf('cherry' in m)
         self.failIf('mint' in m)
 
+    def test_packages(self):
+        '''sandbox package/repository handling'''
+
+        self.assertRaises(ValueError, OSLib.inst.package_description, 'nonexisting')
+        self.assertEqual(OSLib.inst.package_installed('nonexisting'), False)
+
+        self.assertEqual(OSLib.inst.package_description('mesa-vanilla'),
+            fake_pkginfo['default']['mesa-vanilla']['description'])
+        self.assertEqual(OSLib.inst.is_package_free('mesa-vanilla'), False)
+
+        self.assertRaises(ValueError, OSLib.inst.package_description, 'foo-y1')
+        self.assertEqual(OSLib.inst.package_installed('foo-y1'), False)
+        self.assertEqual(OSLib.inst.repository_enabled('http://vaportech.com/x1'), False)
+
+        OSLib.inst.add_repository('http://vaportech.com/x1')
+        self.assertEqual(OSLib.inst.repository_enabled('http://vaportech.com/x1'), True)
+
+        self.assertEqual(OSLib.inst.package_description('foo-y1'),
+            fake_pkginfo['http://vaportech.com/x1']['foo-y1']['description'])
+        self.assertEqual(OSLib.inst.package_installed('foo-y1'), False)
+        self.assertEqual(OSLib.inst.is_package_free('foo-y1'), True)
+        OSLib.inst.install_package('foo-y1', None)
+        self.assertEqual(OSLib.inst.package_installed('foo-y1'), True)
+
+        # test exception raising
+        OSLib.inst.pending_install_remove_exception = 'moo!'
+        self.assertRaises(SystemError, OSLib.inst.install_package, 'foo-y1', None)
+        OSLib.inst.install_package('foo-y1', None)
+        OSLib.inst.pending_install_remove_exception = 'moo!'
+        self.assertRaises(SystemError, OSLib.inst.remove_package, 'foo-y1', None)
+        self.assertEqual(OSLib.inst.package_installed('foo-y1'), True)
+
+        OSLib.inst.remove_repository('foo')
+
+        OSLib.inst.remove_repository('http://vaportech.com/x1')
+        self.assertEqual(OSLib.inst.repository_enabled('http://vaportech.com/x1'), False)
+        self.assertEqual(OSLib.inst.package_installed('foo-y1'), True)
+
+        OSLib.inst.remove_package('foo-y1', None)
+        self.assertEqual(OSLib.inst.package_installed('foo-y1'), False)
+
+        self.assertRaises(ValueError, OSLib.inst.package_description, 'foo-y1')
+        self.assertEqual(OSLib.inst.package_installed('foo-y1'), False)
+        self.assertEqual(OSLib.inst.package_description('mesa-vanilla'),
+            fake_pkginfo['default']['mesa-vanilla']['description'])
+
+        OSLib.inst.reset_packages()
+
+#-------------------------------------------------------------------#
+# Test XML-RPC consistency test
+
+class TestXMLRPCDriverDBConsistency(unittest.TestCase):
+    def setUp(self):
+        start_driverdb_server()
+        self.xmlrpc_client = xmlrpclib.ServerProxy('http://localhost:80080')
+
+    def tearDown(self):
+        stop_driverdb_server()
+
+    def test_consistency(self):
+        '''XML-RPC driver DB server answers are consistent with fake_db'''
+
+        # TODO: kernel_ver, and architecture specific tests
+        self.assertEqual(self.xmlrpc_client.query('20080407', '0', {
+            'os_name': 'Foonux', 'os_version': '42', 'system_vendor': '',
+            'system_product': '', 'components': [
+                'modalias:pci:v00001001d00003D11sv01sd02bc03sc00i00',
+                'modalias:dmi:123456']}),
+            ['20080407', '0', {
+                'modalias:pci:v00001001d00003D11sv01sd02bc03sc00i00': [
+                    {'driver_type': 'kernel_module', 'jockey_handler':
+                     'VanillaGfxHandler', 'description': {'C': 'desc'}}, 
+                    {'driver_type': 'kernel_module', 'foobar':
+                    'IHaveNoHandler', 'description': {'C': 'desc'}}],
+                'modalias:dmi:123456': []}
+            ]
+        )
+
+        self.assertEqual(self.xmlrpc_client.query('20080407', '0', {
+            'os_name': 'RedSock', 'os_version': '2.0', 'system_vendor': '',
+            'system_product': '', 'components': [
+                'modalias:pci:v00001001d00003D11sv01sd02bc03sc00i00']}),
+            ['20080407', '0', {
+                'modalias:pci:v00001001d00003D11sv01sd02bc03sc00i00': [
+                    {'driver_type': 'kernel_module', 'jockey_handler':
+                     'Vanilla3DHandler', 'description': {'C': 'desc'},
+                     'repository': 'http://nonfree.redsock.com/addons'}]
+            }]
+        )
+
+        self.assertEqual(self.xmlrpc_client.query('20080407', '0', {
+            'os_name': 'Foonux', 'os_version': '40', 'system_vendor': '',
+            'system_product': '', 'components': [
+                'modalias:pci:v00001001d00003D11sv01sd02bc03sc00i00',
+                'modalias:othertype:foobar12']}),
+            ['20080407', '0', {
+                'modalias:pci:v00001001d00003D11sv01sd02bc03sc00i00': [],
+                'modalias:othertype:foobar12': []}
+            ]
+        )
+
+        self.assertEqual(self.xmlrpc_client.query('20080407', '0', {
+            'os_name': 'Foonux', 'os_version': '42', 'system_vendor':
+            'VaporTech Inc.', 'system_product': 'Virtualtop X-1', 
+            'components': ['modalias:pci:v0000FFF0d00000001sv01sd01bc06sc01i00']}),
+            ['20080407', '0', {
+                'modalias:pci:v0000FFF0d00000001sv01sd01bc06sc01i00': [
+                    {'driver_type': 'kernel_module', 'kernel_module':
+                     'chocolate', 'description': {'C': 'VaporTech choc handler', 'de_DE':
+                         'German description', 'cs': 'Czech description'},
+                     'long_description': {'C': 'Multi\nline'},
+                     'driver_vendor': 'VaporTech', 'package': 'spam-x1', 'repository':
+                     'http://vaportech.com/x1', 'version': '1.1-foo42-vt3'},
+                    {'driver_type': 'kernel_module', 'jockey_handler':
+                     'NoDetectMod', 'driver_vendor': 'White Hat',
+                     'description': {'C': 'WhiteHat ndmod'}},
+                    {'driver_type': 'kernel_module', 'jockey_handler':
+                     'NoDetectMod', 'driver_vendor': 'Black Hat',
+                     'description': {'C': 'BlackHat ndmod'}},
+                    {'driver_type': 'kernel_module', 'kernel_module':
+                     'mint', 'driver_vendor': 'UbuOne',
+                     'description': {'C': 'UbuOne mintmod'}},
+                    {'driver_type': 'kernel_module', 'kernel_module':
+                     'mint', 'driver_vendor': 'UbunTwo',
+                     'description': {'C': 'UbunTwo mintmod'},
+                     'recommended': True},
+                    {'driver_type': 'kernel_module', 'kernel_module':
+                     'apple', 'driver_vendor': 'Apple', 'description':
+                     {'C': 'Unknown local kmod'}, 'free': False, 
+                     'license': 'Kills kittens', 'package': 'pretzel'},
+                ]}
+            ]
+        )
+
+    def test_invalid_query(self):
+        '''XML-RPC driver DB server/client survive incomplete queries'''
+
+        self.assertEqual(self.xmlrpc_client.query('20080407', '0', {}), 
+            ['20080407', '0', {}])
+        self.assertEqual(self.xmlrpc_client.query('20080407', '0', {
+            'components': ['modalias:pci:foo']}),
+            ['20080407', '0', {}])
+        self.assertEqual(self.xmlrpc_client.query('20080407', '0', {
+            'os_name': 'Foonux', 'components': ['modalias:pci:foo']}),
+            ['20080407', '0', {}])
+
+    def test_introspection(self):
+        '''XML-RPC driver DB server supports expected methods'''
+
+        self.assert_('query' in self.xmlrpc_client.system.listMethods())
+        # methodSignature does not work with Python XML-RPC server
+
+
 #-------------------------------------------------------------------#
 # test handlers
 
